function [x, L] = dmlr1(z, h, SN, Lbd, bi)
% Deconvolution de processus Bernoulli-Gaussiens par maximum de
% vraisemblance marginal. Forme covariance. Exploration SMLR des
% sequences de Bernoulli par cellules de 1 echantillon.
%
% Algorithme "turbo". Version normalisee.
%
% Reference: 	Goussard, Demoment, Idier
%		A new algorithm for iterative deconvolution of
%		sparse spike trains
%		ICASSP-90
%		(After correction of equation 24. Epsilon factor of
%		second term of rhs only) Pour le calcul du critere
%
%		Document Idier & Champagnat pour l'algorithme de base
%
% Syntax:	[x, L] = dmlr1(z, h, SN, Lbd, bi)
%
%		z: Observed sequence. Continuous data.
%		h: Impulse response. Continuous data.
%		SN: Signal-to noise ratio, in dB.
%		Lbd: Bernoulli parameter.
%		bi: Initial Bernoulli sequence. Default value: zero sequence
%
%		x: Restored Bernoulli-Gaussian sequence.
%		L: Valeur finale du critere
%
% Other custom external procedures called:
%
%		mlr1
%		reinit
%		o1modL
%		o1moddCG
%

%	TESTS

%	Sampling periods

Te_z = z(2, 1) - z(1, 1);
Te_h = h(2, 1) - h(1, 1);
if abs(Te_z - Te_h) >= 10*eps
    fprintf('\nERROR "dmlr1": Check sampling frequencies of signals\n')
    return
end

%	Bernoulli parameter

if (Lbd < 0) | (Lbd > 1)
    fprintf('\nERROR "dmlr1": The Bernoulli parameter must belong to [0,1]\n')
    return
end

%	Size of vector quantities

M = size(z); P = M(1);		% Number of samples in z
M = size(h); p = M(1);		% Number of samples in h
N = P + p - 1;				% Number of samples in x

%	size of bi

if exist('bi') == 1		% If bi has been specified
    M = size(bi);		% Size of bi
    if M(1) ~= N
	fprintf('\nERROR "dmlr1": Inconsistent size of initial Bernoulli')
	fprintf(' sequence\n')
	return
    end
end

%	INITIALIZATION OF VARIANCES

fprintf('\nComputing variances...\n')

rz = (z(:, 2)' * z(:, 2)) / P;	% Mean power of the observations
rn = rz / (1 + 10^(SN/10));	% Mean power of the observation noise
ry = rz - rn;			% Mean power of the noiseless output
rx = ry / (Lbd * (h(:, 2)'*h(:, 2)));
				% Variance of the Gaussian part of the
				% input

%	DECONVOLUTION

if exist('bi') == 1		% If bi has been specified
    [x, L] = mlr1(z, h, rx, rn, Lbd, bi);
else				% If bi has not been specified
    [x, L] = mlr1(z, h, rx, rn, Lbd);
end

return
