function [x, L] = mlr1(z, h, rx, rn, Lbd, bi)
% Deconvolution de processus Bernoulli-Gaussiens par maximum de
% vraisemblance marginal. Forme covariance. Exploration SMLR des
% sequences de Bernoulli par cellules de 1 echantillon.
%
% Algorithme "turbo". Version normalisee.
%
% Reference: 	Goussard, Demoment, Idier
%		A new algorithm for iterative deconvolution of
%		sparse spike trains
%		ICASSP-90
%		(After correction of equation 24. Epsilon factor of
%		second term of rhs only) Pour le calcul du critere
%
%		Document Idier & Champagnat pour l'algorithme de base
%
% Syntax:	[x, L] = mlr1(z, h, rx, rn, Lbd, bi)
%
%		z: Observed sequence. Continuous data.
%		h: Impulse response. Continuous data.
%		rx: Variance of the Gaussian part of the input
%		rn: Variance of the observation noise
%		Lbd: Bernoulli parameter.
%		bi: Initial Bernoulli sequence. Default value: zero sequence
%
%		x: Restored Bernoulli-Gaussian sequence.
%		L: Valeur finale du critere
%
% Other custom external procedures called:
%
%		reinit
%		o1modL
%		o1moddCG
%

%	TESTS

%	Sampling periods

Te_z = z(2, 1) - z(1, 1);
Te_h = h(2, 1) - h(1, 1);
if abs(Te_z - Te_h) >= 10*eps
    fprintf('\nERROR "mlr1": Check sampling frequencies of signals\n')
    return
end

%	Bernoulli parameter

if (Lbd < 0) | (Lbd > 1)
    fprintf('\nERROR "mlr1": The Bernoulli parameter must belong to [0,1]\n')
    return
end

%	Variances

if (rx < 0) | (rn <= 0)
    fprintf('\nERROR "mlr1": Variances must be greater than zero\n')
    return
end

%	Size of vector quantities

M = size(z); P = M(1);		% Number of samples in z
M = size(h); p = M(1);		% Number of samples in h
N = P + p - 1;			% Number of samples in x

%	size of bi

if exist('bi') == 1		% If bi has been specified
    M = size(bi);		% Size of bi
    if M(1) ~= N
	fprintf('\nERROR "mlr1": Inconsistent size of initial Bernoulli')
	fprintf(' sequence\n')
	return
    end
end

%	INITIALISATION (pour une sequence de Bernoulli nulle)

fprintf('Initializing internal variables...\n')

x = zeros(N, 3);		% Initialization of x
x(:,1) = (z(1,1)-h(p, 1):Te_z:z(P, 1)-h(1, 1))';


d = [];				% Vecteur ligne contenant les indices des
				% echantillons non nuls de la sequence de Bernoulli
Cinv = [];			% Matrice inv(C) de l'algorithme
G = [];				% Matrice G de l'algorithme
% Gt_z = G' * z(:, 2);		% Vecteur Gt z
Gt_z = [];

%	Normalisation de la reponse impulsionnelle

h_norm = sqrt(rx / rn) * h(:, 2);

%	Initialization of the criterion

L = - (z(:, 2)' * z(:, 2)) / rn - P * log(rn) + 2 * N * log(1-Lbd);

%	Storage of initial values for possible reinitialization

L0 = L;				% Criterion L

%	UPDATE OF d, inv(C), G, G' z AND L TO ACCOUNT FOR bi, IF PRESENT

if exist('bi') == 1		% If bi has been specified

    fprintf('Accounting for initial Bernoulli sequence...\n')
    d = find(bi > 0.5)';	% Determination du vecteur ligne d correspondant
	[Cinv, G, Gt_z, L] = reinit(d, L0, z(:, 2), h_norm, Lbd, rn);
				% Initialization of d, inv(C), G, G' z and L
end

%	DECONVOLUTION

Lold = L - 1;			% Old value of criterion
Iter = 0;			% Number of iterations

while Lold < L			% Convergence test

    Lold = L;			% Store current value of criterion
    Iter = Iter + 1;		% Increment iteration counter
    fprintf('\nIteration number %g\n', Iter)
    fprintf('Current value of the criterion: %g\n', L)
    fprintf('Scanning input sequence...\n')
    				% Messages

    for i = 1:N			% Scanning of the input sequence
	[Lnew(i), flag(i)] = o1modL(L, i, d, Cinv, G, Gt_z, z(:, 2), ...
	h_norm, Lbd, rn);	% Modified value of the criterion
    end
    
    if sum(flag) ~= 0		% Numerical instability detected

	fprintf('\n"mlr1": Numerical instability detected.\n')
	fprintf('          Reinitialization in progress...\n')
	
	[Cinv, G, Gt_z, L] = reinit(d, L0, z(:, 2), h_norm, Lbd, rn);
				% Initialization of d, inv(C), G, G' z and L

	fprintf('           Reinitialization complete. Resuming')
	fprintf(' restoration at iteration %g\n', Iter)
	Lold = L-1;		% Update of Lold, to make sure that the
				% procedure is resumed
	Iter = Iter - 1;	% Adequate modification of the iteration count
    else

	fprintf('Comparing criterion values...\n')
	[L,I] = max(Lnew);	% Max new value of the criterion, and
    				% corresponding index

	if L > Lold		% Increase of the criterion
	    fprintf('Updating internal variables for next iteration...\n')
	    [d, Cinv, G, Gt_z] = o1moddCG(I, d, Cinv, G, Gt_z, z(:, 2), ...
	    h_norm, Lbd);
				% Update of d, inv(C), G, et G' z
	end

    end

end

L = Lold;			% Valeur finale du critere

%	Estimation of the amplitudes

fprintf('\nEstimating the amplitudes...\n\n')
	[dr, dc] = size(d);	% Calcul necessaire pour eviter les problemes
				% lorsque d est de taille 1,1
x(d', 3) = ones(dc, dr);
				% Expansion de la sequence de Bernoulli
x(d', 2) = sqrt(rx / rn) * (Cinv * Gt_z);

				% Calcul des amplitudes
return
