function [Lnew, flag] = o1modL(Lcur, i, d, Cinv, G, Gt_z, z, h, Lbd, rn)
% Fonction permettant le calcul recursif du critere de vraisemblance
% marginale lorsque la sequence de Bernoulli subit une modification
% d'ordre 1 (1 seul echantillon modifie).
%
% Reference: 	Goussard, Demoment, Idier
%		A new algorithm for iterative deconvolution of
%		sparse spike trains
%		ICASSP-90
%		(After correction of equation 24. Epsilon factor of
%		second term of rhs only) Pour le calcul du critere
%
%		Document Idier & Champagnat pour l'algorithme de base
%
% Syntax:	[Lnew, flag] = o1modL(Lcur, i, d, Cinv, G, Gt_z, z, h, Lbd, rn)
%
%		Lcur: valeur courante du critere.
%		i: indice de l'echantillon modifie
%		d: vecteur ligne contenant les indices des echantillons non nuls de
%		   la sequence de Bernoulli courante
%		Cinv: matrice inv(C) de l'algorithme
%		G: matrice G de l'algorithme
%		Gt_z: Produit G' * z
%		z: vecteur contenant les observations
%		h: vecteur contenant la reponse impulsionnelle normalisee
%		Lbd: Parametre de la sequence de  Bernoulli
%		rn: variance du bruit d'observation
%
%		Lnew: Nouvelle valeur du critere.
%		flag: Flag d'erreur (instabilite numerique)
%		      0: OK; 1: Probleme.
%

%	TESTS

%	Size of vectors and matrices

[dr, dc] = size(d);		% Dimension of d
[Cr, Cc] = size(Cinv);		% Dimension of inv(C)
[Gr, Gc] = size(G);		% Dimension of G
[Gt_zr, Gt_zc] = size(Gt_z);% Dimension of Gt_z
[zr, zc] = size(z);		% Dimension of z
[hr, hc] = size(h);		% Dimension of h

P = zr; p = hr; N = P+p-1;	% Dimensions respectives des observations, de la
				% RI et de la sequence d'entree 

% 	Tests

if (dc ~= Cr) | (Cr ~= Cc) | (Cc ~= Gc)
				% If some dimensions are different
    fprintf('ERROR "o1modL": Dimensions of internal variables of the algorithm')
    fprintf(' must agree\n')
    return
end

if (dc ~= 0) & ((Gr ~= zr) | (Gc ~= Gt_zr))
				% Test supplementaire lorsque la sequence de
				% Bernoulli n'est pas identiquement nulle
    fprintf('ERROR "o1modL": Dimensions of internal variables of the algorithm')
    fprintf(' must agree\n')
    return
end

%	Bernoulli parameter

if (Lbd < 0) | (Lbd > 1)
    fprintf('\nERROR "o1modL": The Bernoulli parameter must belong to [0,1]\n')
    return
end

%	Variances

if rn <= 0
    fprintf('\nERROR "o1modL": Noise variance must be greater than zero\n')
    return
end

% Index of modified Bernoulli sample

if (i < 1) | (i > N)
    fprintf('ERROR "o1modL": Index of modified Bernoulli sample out of range\n')
    return
end

%	COMPUTATION OF THE NEW CRITERION VALUE

%	Initialisations

Epsilon = 1;			% Signe de la substitution

i_deb = p-i+1; i_fin = N-i+1;	% Indices de la RI correspondant a la premiere
				% et a la derniere coordonnee du vecteur g
				% complet (de taille P)
p_deb = 1; p_fin = P;		% Position de ces echantillons dans le vecteur
				% g complet
if i_deb <= 1			% Elimination des zeros en tete du vecteur g complet
    i_deb = 1;
    p_deb = i-p+1;
end

if i_fin >= p			% Elimination des zeros en queue du vecteur g complet
    i_fin = p;
    p_fin = i;
end

g_u = h(i_deb:i_fin);		% Partie utile du vecteur g complet

zBg = z(p_deb:p_fin)' * g_u;	% Initialisation de z' inv(B) g
gBg = g_u' * g_u;		% Initialisation de g' inv(B) g

%	Calcul des termes correcteurs si la sequence de Bernoulli est non vide

if dc ~= 0			% Si la sequence de Bernoulli est non vide

    if any(d == i)
	Epsilon = -1;		% Mise a jour du signe de la substitution
    end

    Gtg	= G(p_deb:p_fin, :)' * g_u;
				% Calcul de G' g
    CinvGtg = Cinv * Gtg;	% Calcul de inv(C) G' g
    zBg = zBg - Gt_z' * CinvGtg;% Mise a jour de z' inv(B) g
    gBg = gBg - Gtg' * CinvGtg;	% Mise a jour de g' inv(B) g
end

%	Calcul de la nouvelle valeur du critere

rho = Epsilon + gBg;		% Quantite intermediaire
Lnew = Lcur + zBg^2 / (rho * rn) - log(Epsilon * rho) - ...
Epsilon*2*log((1-Lbd)/Lbd);	% Nouvelle valeur du critere

flag = 0;			% Set error flag
if Epsilon*rho <= 0, flag = 1; end
				% Numerical problem

return
