function [dnew, Cinvnew, Gnew, Gt_znew] = o1moddCG(i, d, Cinv, G, Gt_z, ...
z, h, Lbd)
% Fonction permettant la mise a jour recursive des quantites d, inv(C), G et
% G' z lorsque la sequence de Bernoulli subit
% une modification d'ordre 1 (1 seul echantillon modifie).
% Version normalisee
%
% Reference: 	Goussard, Demoment, Idier
%		A new algorithm for iterative deconvolution of
%		sparse spike trains
%		ICASSP-90
%		(After correction of equation 24. Epsilon factor of
%		second term of rhs only) Pour le calcul du critere
%
%		Document Idier & Champagnat pour l'algorithme de base
%
% Syntax:	[dnew, Cinvnew, Gnew, Gt_znew] = o1moddCG(i, d, Cinv, G, Gt_z, 
%		z, h, Lbd)
%
%		i: indice de l'echantillon modifie
%		d: vecteur ligne contenant les indices des echantillons non nuls de
%		   la sequence de Bernoulli courante
%		Cinv: matrice inv(C) de l'algorithme
%		G: matrice G de l'algorithme
%		Gt_z: produit G' * z
%		z: vecteur contenant les observations
%		h: vecteur contenant la reponse impulsionnelle normalisee
%		Lbd: Parametre de la sequence de  Bernoulli
%
%		dnew: nouvelle valeur du vecteur ligne contenant les indices des
%		   	  echantillons non nuls de la sequence de Bernoulli courante
%		Cinvnew: nouvelle valeur de la matrice inv(C) de l'algorithme
% 		Gnew: nouvelle valeur de la matrice C de l'algorithme
%		Gt_znew: nouvelle valeur du produit G' * z
%

%	TESTS

%	Size of vectors and matrices

[dr, dc] = size(d);		% Dimension of d
[Cr, Cc] = size(Cinv);		% Dimension of inv(C)
[Gr, Gc] = size(G);		% Dimension of G
[Gt_zr, Gt_zc] = size(Gt_z);% Dimension of Gt_z
[zr, zc] = size(z);		% Dimension of z
[hr, hc] = size(h);		% Dimension of h

P = zr; p = hr; N = P+p-1;	% Dimensions respectives des observations, de la
				% RI et de la sequence d'entree 

% 	Tests

if (dc ~= Cr) | (Cr ~= Cc) | (Cc ~= Gc)
				% If some dimensions are different
    fprintf('ERROR "o1moddCG": Dimensions of internal variables of the algorithm')
    fprintf(' must agree\n')
    return
end

if (dc ~= 0) & ((Gr ~= zr) | (Gc ~= Gt_zr))
				% Test supplementaire lorsque la sequence de
				% Bernoulli n'est pas identiquement nulle
    fprintf('ERROR "o1moddCG": Dimensions of internal variables of the algorithm')
    fprintf(' must agree\n')
    return
end

%	Bernoulli parameter

if (Lbd < 0) | (Lbd > 1)
    fprintf('\nERROR "o1moddCG": The Bernoulli parameter must belong to [0,1]\n')
    return
end

% Index of modified Bernoulli sample

if (i < 1) | (i > N)
    fprintf('ERROR "o1moddCG": Index of modified Bernoulli sample out of range\n')
    return
end

%	MISES A JOUR

%	Signe de la substitution

if any(d == i)			% SUPPRESSION D'UN ECHANTILLON
	
    index = find(d == i);	% Indice de l'echantillon a supprimer dans d,
				% inv(C) et G
    dnew = d; dnew(index) = [];	% Suppression de l'echantillon correspondant dans d
    Gnew = G; Gnew(:, index) = [];
				% Suppression de la colonne correspondante dans G
    Gt_znew = Gt_z; Gt_znew(index) = [];
				% Suppression de l'echantillon correspondant dans
				% G' z
    Cinvnew = Cinv;		% Initialisation de Cinvnew
    alpha = Cinv(index, index);
    v = Cinv(:, index); v(index) = [];
				% Intermediaires de calcul pour la mise a jour de
				% inv(C)
    Cinvnew(:, index) = [];	% Suppression de la colonne correspondante dans 
				% inv(C)
    if dc ~= 1, Cinvnew(index, :) = []; end
				% Suppression de la ligne correspondante dans inv(C)
				% si Cinvnew n'est pas deja vide
    Cinvnew = Cinvnew - (v/alpha) * v';
				% Nouvelle valeur de inv(C)
	
else				% ADDITION D'UN ECHANTILLON

%	Initialisations

    i_deb = p-i+1; i_fin = N-i+1;
				% Indices de la RI correspondant a la premiere
				% et a la derniere coordonnee du vecteur g
				% complet (de taille P)
    p_deb = 1; p_fin = P;	% Position de ces echantillons dans le vecteur
				% g complet
    if i_deb <= 1		% Elimination des zeros en tete du vecteur g complet
	i_deb = 1;
	p_deb = i-p+1;
    end

    if i_fin >= p		% Elimination des zeros en queue du vecteur g complet
	i_fin = p;
	p_fin = i;
    end

    g_u = h(i_deb:i_fin);	% Partie utile du vecteur g complet


%	Mises a jour

    dnew = [d i];		% Nouvelle valeur du vecteur d
	
    Gnew = [G zeros(P, 1)];	% Nouvelle valuer de la matrice G
    Gnew(p_deb:p_fin, dc+1) = g_u;
	
    Gt_znew = [Gt_z; g_u'*z(p_deb:p_fin)];
				% Nouvelle valeur du vecteur G' z
	
    gBg = g_u' * g_u;		% Initialisation de g' inv(B) g

    if dc ~= 0			% Si la sequence de Bernoulli est non vide
	Gtg = G(p_deb:p_fin, :)' * g_u;
				% Calcul de G' g
	CinvGtg = Cinv * Gtg;
				% Calcul de inv(C) G' g
	gBg = gBg - Gtg' * CinvGtg;
				% Mise a jour de g' inv(B) g

    else			% Si la sequence de Bernoulli est non vide
	CinvGtg = [];		% inv(C) G' g est vide
    end
	
				% Intermediaires de calcul pour la mise a jour de
				% inv(C)
    alpha = 1 / (1 + gBg);
    v = - alpha * CinvGtg;
    Cinvnew = Cinv + (v / alpha) * v';
				% Mise a jour de inv(C)
    Cinvnew = [Cinvnew v; v' alpha];
	
end

return

