function [Cinv, G, Gt_z, L] = reinit(df, L0, z, h, Lbd, rn)
% Reinitialization of the algorithm when a numerical instability has
% been detected. Given the value L for a zero Bernoulli sequence
% and the current (nonzero) Bernoulli sequence, d, inv(C), G and G' z are
% updated recursively using order one modifications. These modifications
% require only additions of pulses, amd are therefore numerically stable.
%
% Syntax:	[Cinv, G, Gt_z, L] = reinit(df, L0, z, h, Lbd, rn)
%
%		df: Indices of the nonzero samples of the final Bernoulli sequence
%		   (row vector)
%		L0: Initial value of L (zero Bernoulli sequence)
%		z: vecteur contenant les observations
%		h: vecteur contenant la reponse impulsionnelle normalisee
%		Lbd: Parametre de la sequence de  Bernoulli
%		rn: variance du bruit d'observation
%
%		Cinv: matrice inv(C) de l'algorithme
%		G: matrice G de l'algorithme
%		Gt_z: produit G' * z
%		L: Valeur du critere
%

%	TESTS

%	Size of vectors and matrices

[zr, zc] = size(z);		% Dimension of z
[hr, hc] = size(h);		% Dimension of h
[dr, dc] = size(df);		% Dimension de df
P = zr; p = hr; N = P + p - 1;
				% Dimensions respectives des observations, de la
				% RI et de la sequence d'entree 

% 	Tests

%	Bernoulli parameter

if (Lbd < 0) | (Lbd > 1)
    fprintf('\nERROR "reinit": The Bernoulli parameter must belong to [0,1]\n')
    return
end

%	Variance

if rn <= 0
    fprintf('\nERROR "reinit": Noise variance must be greater than zero\n')
    return
end

%	REINITIALIZATION

d = [];				% Vecteur ligne contenant les indices des
				% echantillons non nuls de la sequence de Bernoulli
Cinv = [];			% Matrice inv(C) de l'algorithme
G = [];				% Matrice G de l'algorithme
Gt_z = G' * z;			% Vecteur Gt z
L = L0;				% Critere

for i = 1:dc			% For each element of df

    [L, flag] = o1modL(L, df(i), d, Cinv, G, Gt_z, z, h, Lbd, rn);
				% New value of the criterion
    [d, Cinv, G, Gt_z] = o1moddCG(df(i), d, Cinv, G, Gt_z, z, h, Lbd);
				% New value of d, Cinv, G, Gt_z
end

return
